import os
import sys
import subprocess
import urllib.request

VSCODE_INSTALLER_URL = "https://code.visualstudio.com/sha/download?build=stable&os=win32-x64-user"

def download_vscode_installer(download_path):
    # URL for the Visual Studio Code installer (system installer for 64-bit Windows)
    installer_url = VSCODE_INSTALLER_URL
    
    # Path to save the installer
    installer_path = os.path.join(download_path, "VSCodeUserSetup-x64.exe")
    
    print(f"Downloading Visual Studio Code installer to {installer_path}...")
    try:
        urllib.request.urlretrieve(installer_url, installer_path)
        print("Download complete.")
    except Exception as e:
        print(f"Failed to download Visual Studio Code installer: {e}")
        sys.exit(1)
    
    return installer_path

def install_vscode():
    download_path = r"C:\Temp"
    os.makedirs(download_path, exist_ok=True)
    vscode_installer_path = download_vscode_installer(download_path)
    print(f"Visual Studio Code installer downloaded to: {vscode_installer_path}")

    # Command to silently install VS Code
    #command = f'"{vscode_installer_path}" /silent /mergetasks=!runcode'
    command = f'"{vscode_installer_path}" /mergetasks=!runcode'

    try:
        print("Starting Visual Studio Code installation...")
        subprocess.run(command, shell=True, check=True)
        print("Installation complete.")
    except subprocess.CalledProcessError as e:
        print(f"Installation failed: {e}")
        sys.exit(1)

if __name__ == "__main__":
    install_vscode()


import json
import argparse
from datetime import datetime
import subprocess
import configparser
import os
import configparser
import shlex

import configparser

class CustomConfigParser(configparser.ConfigParser):
    def optionxform(self, optionstr):
        return optionstr  # Retain the original case

    def write(self, fp, spaces=False):
        """Write an .ini-format representation of the configuration state."""
        if spaces:
            delimiter = " = "
        else:
            delimiter = "="
        for section in self._sections:
            fp.write(f"[{section}]\n")
            for key, value in self._sections[section].items():
                if key != "__name__":
                    fp.write(f"{key}{delimiter}{value}\n")
            fp.write("\n")

PROJECT_DIR = './'
CHANGELOG_FILE = os.path.join(PROJECT_DIR, 'Changelog.json')
README_FILE = os.path.join(PROJECT_DIR, 'README.md')
INI_FILE_PATH = os.path.join(PROJECT_DIR, 'Config', 'DefaultGame.ini')

def load_changelog():
    with open(CHANGELOG_FILE, 'r') as file:
        return json.load(file)

def save_changelog(changelog):
    changelog[-1]['ReleaseDate'] = datetime.now().strftime('%Y-%m-%d')
    changelog[-1]['Commit'] = get_commit_hash()
    with open(CHANGELOG_FILE, 'w') as file:
        json.dump(changelog, file, indent=4)

def update_readme(changelog):
    with open(README_FILE, 'a') as file:
        entry = changelog[-1]
        version = entry.get('Version', 'Unknown')
        prerelease = entry.get('PrereleaseType', 'Release')
        prerelease = ""
        if 'PrereleaseType' in entry:
            prerelease = f" - {entry['PrereleaseType']}"
        release_date = entry.get('ReleaseDate', 'Unknown')
        commit = entry.get('Commit', 'Unknown')
        file.write(f"\n\n### Version {version} {prerelease}\n\n")
        file.write(f"**Release Date**: {release_date}\n\n")
        file.write(f"**Commit**: {commit}\n\n")
        file.write("**Changes**\n\n")
        for change in entry['Changes']:
            file.write(f"- {change}\n")

def get_commit_hash():
    return subprocess.check_output(['git', 'rev-parse', 'HEAD']).strip().decode()

def increment_version(version):
    major, minor, patch = map(int, version.split('.'))
    patch += 1
    return f"{major}.{minor}.{patch}"

def add_version():
    changelog = load_changelog()
    latest_version = changelog[-1]['Version']
    new_version = increment_version(latest_version)
    new_entry = {
        "Version": new_version,
        "PrereleaseType": "Alpha",
        "ReleaseDate": datetime.now().strftime('%Y-%m-%d'),
        "Commit": get_commit_hash(),
        "Changes": []
    }
    changelog.append(new_entry)
    save_changelog(changelog)

def escape_input(user_input):
    return shlex.quote(user_input)

def add_change(change):
    changelog = load_changelog()
    changelog[-1]['Changes'].append(change)
    save_changelog(changelog)

def update_ini():
    changelog = load_changelog()
    config = CustomConfigParser(strict=False)
    config.read(INI_FILE_PATH)

    section = '/Script/EngineSettings.GeneralProjectSettings'
    if section in config:
        config[section]['ProjectVersion'] = changelog[-1]['Version']
    else:
        raise Exception(f"Section {section} not found in {INI_FILE_PATH}")

    # Read the original file content
    with open(INI_FILE_PATH, 'r') as file:
        original_content = file.readlines()

    # Write the updated content back to the file
    with open(INI_FILE_PATH, 'w') as file:
        section_found = False
        for line in original_content:
            if line.strip().startswith(f"[{section}]"):
                section_found = True
            if section_found and line.strip().startswith("ProjectVersion"):
                file.write(f"ProjectVersion={changelog[-1]['Version']}\n")
                section_found = False  # Only update the first occurrence
            else:
                file.write(line)

def main():
    parser = argparse.ArgumentParser(description='Manage changelog')
    parser.add_argument('--add-version', action='store_true', help='Add an incremented version to the changelog')
    parser.add_argument('--add-change', type=str, help='Add a change to the most recent version')
    parser.add_argument('--update-readme', action='store_true', help='Append changes to README.md')
    parser.add_argument('--update-ini', action='store_true', help='Copy the most recent version number Config/DefaultGame.ini')

    args = parser.parse_args()

    if args.add_version:
        add_version()
    if args.add_change:
        add_change(args.add_change)
    if args.update_readme:
        changelog = load_changelog()
        update_readme(changelog)
    if args.update_ini:
        update_ini()

if __name__ == '__main__':
    main()


import os
import shutil
import json
import platform
from datetime import datetime

UNREAL_PATH_KEY = "UNREAL_PATH"
PROJECT_NAME_KEY = "PROJECT_NAME"
WORKSPACE_NAME_KEY = "WORKSPACE_NAME"
CHANGELOG_FILENAME_KEY = "CHANGELOG_FILENAME"

CONFIG_FILENAME = "config.upm"
GITIGNORE_SRCFILE = "gitignore.upm"
GITIGNORE_DSTFILE = ".gitignore"
UPM_DIR = os.path.normpath(os.path.abspath(os.path.dirname(__file__)))

def clean(args):
    upm_dir_path = os.path.join(args.dir, "upm")
    config_file_path = os.path.join(args.dir, CONFIG_FILENAME)
    gitignore_file_path = os.path.join(args.dir, GITIGNORE_DSTFILE)
    changelog_file_path = os.path.join(args.dir, args.changelog)

    # Remove the "upm" directory
    if os.path.exists(upm_dir_path):
        shutil.rmtree(upm_dir_path)
        print(f"Removed directory: {upm_dir_path}")
    else:
        print(f"Directory not found, skipping: {upm_dir_path}")

    # Remove the config file
    if os.path.exists(config_file_path):
        os.remove(config_file_path)
        print(f"Removed file: {config_file_path}")
    else:
        print(f"File not found, skipping: {config_file_path}")

    # Remove the .gitignore file
    if os.path.exists(gitignore_file_path):
        os.remove(gitignore_file_path)
        print(f"Removed file: {gitignore_file_path}")
    else:
        print(f"File not found, skipping: {gitignore_file_path}")

    # Remove the changelog file
    if os.path.exists(changelog_file_path):
        os.remove(changelog_file_path)
        print(f"Removed file: {changelog_file_path}")
    else:
        print(f"File not found, skipping: {changelog_file_path}")

    print(f"Config cleanup complete. Run 'upm setup --clean' from {args.dir} to remove generated setup files.")

def load_config_template(args):
    if args.file:
        with open(args.file, 'r') as f:
            return json.load(f)
    else:
        with open(os.path.join(UPM_DIR, CONFIG_FILENAME), 'r') as f:
            return json.load(f)

def config(args):
    if args.clean:
        clean(args)
        return

    shutil.copytree(
        UPM_DIR,
        os.path.join(args.dir, "upm"),
        dirs_exist_ok=True
        )
    print(f"Copied UPM scripts to {os.path.join(args.dir, 'upm')}")
    
    if not args.nogitignore:
        shutil.copy(
            os.path.join(UPM_DIR, GITIGNORE_SRCFILE),
            os.path.join(args.dir, GITIGNORE_DSTFILE)
        )
        print ("Copied .gitignore file.")
    else:
        print("Skipping .gitignore file.")
    
    config = load_config_template(args)

    system = platform.system()

    if not (system == 'Windows' or system == 'Darwin'):
        system = 'Linux'
    
    if args.unreal:
        config[system][UNREAL_PATH_KEY] = args.unreal

    if args.project_name:
        config[system][PROJECT_NAME_KEY] = args.project_name
    else:
        config[system][PROJECT_NAME_KEY] = os.path.basename(args.dir)
    
    if args.workspace:
        config[system][WORKSPACE_NAME_KEY] = args.workspace
    else:
        config[system][WORKSPACE_NAME_KEY] = f"{os.path.basename(args.dir)}.code-workspace"

    if os.path.exists(os.path.join(args.dir, CONFIG_FILENAME)):
        print(f"Skipping create config file; UPM config file already exists at {os.path.join(args.dir, CONFIG_FILENAME)}")
    else:
        with open(os.path.join(args.dir, CONFIG_FILENAME), 'w') as f:
            json.dump(config, f, indent=4)
        print(f"Created UPM config file at {os.path.join(args.dir, CONFIG_FILENAME)}")
    
    if os.path.exists(os.path.join(args.dir, args.changelog)):
        print(f"Skipping create changelog file; changelog file already exists at {os.path.join(args.dir, args.changelog)}")
    else:
        if args.changelog:
            config[system][CHANGELOG_FILENAME_KEY] = args.changelog
            changelog_stub = [{
                "Version": "0.0.0",
                "PrereleaseType": "Alpha",
                "ReleaseDate": datetime.now().strftime('%Y-%m-%d'),
                "Changes": [],
            }]
            with open(os.path.join(args.dir, args.changelog), 'w') as f:
                json.dump(changelog_stub, f, indent=4)
                print(f"Created changelog file with v.0.0.0 at {os.path.join(args.dir, args.changelog)}")
    
    print(f"Configuration complete. Navigate to {args.dir} and run 'upm setup' to complete setup.")
    



from setuptools import setup, find_packages

setup(
    name='unreal-project-manager',
    version='0.1.2',
    packages=find_packages(where='.'),
    include_package_data=True,
    install_requires=[
        'python-dotenv'
    ],
    entry_points={
        'console_scripts': [
            'upm=upm.__main__:main'
        ]
    },
    package_data={
        'upm': ['*.upm', '*.txt']
    },
    author='Brandt Frazier',
    author_email='brandt@frazimuth.me',
    description='A lightweight Python module for managing Unreal projects across space, time, and platforms.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/crashtestbrandt/unrealprojectmanager',
    ckassifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent'
    ],
    python_requires='>=3.10'
)